--[[----------------------------------------------------------------------------

  Author : Alexander Stoica
  Creation Date : 08/28/2009
  Last modified : 10/04/2010
  Version : 1.21

----------------------------------------------------------------------------]]--

_AUTO_RELOAD_DEBUG = true

--[[ locals ]]--------------------------------------------------------------]]--

local at_menu_name = "Mixer:Mixer:"
local ct_menu_name = "Mixer:Track:"
local at_keybind_name = "Mixer:Show Automated Sliders:All Tracks - "
local ct_keybind_name = "Mixer:Show Automated Sliders:Current Track - "

--[[ menus, keybindings and notifiers ]]------------------------------------]]--

renoise.tool():add_menu_entry {
  name = at_menu_name .. "Show All Automated Sliders",
  invoke = function() show_automated(false, true) end,
  active = function() return menu_active(false) end
}

renoise.tool():add_keybinding {
  name = at_keybind_name .. "Show All Automated Sliders",
  invoke = function(repeated) show_automated(false, true) end
}

renoise.tool():add_menu_entry {
  name = ct_menu_name .. "Show All Automated Sliders",
  invoke = function() show_automated(true, true) end,
  active = function() return menu_active(true) end
}

renoise.tool():add_keybinding {
  name = ct_keybind_name .. "Show All Automated Sliders",
  invoke = function(repeated) show_automated(true, true) end
}

renoise.tool():add_menu_entry {
  name = at_menu_name .. "Show All Modulated Sliders",
  invoke = function() show_modulated(false, true) end,
  active = function() return menu_active(false) end
}

renoise.tool():add_keybinding {
  name = at_keybind_name .. "Show All Modulated Sliders",
  invoke = function(repeated) show_modulated(false, true) end
}

renoise.tool():add_menu_entry {
  name = ct_menu_name .. "Show All Modulated Sliders",
  invoke = function() show_modulated(true, true) end,
  active = function() return menu_active(true) end
}

renoise.tool():add_keybinding {
  name = ct_keybind_name .. "Show All Modulated Sliders",
  invoke = function(repeated) show_modulated(true, true) end
}

renoise.tool():add_menu_entry {
  name = at_menu_name .. "Show All Automated And Modulated Sliders",
  invoke = function() show_automated_modulated(false) end,
  active = function() return menu_active(false) end
}

renoise.tool():add_keybinding {
  name = at_keybind_name .. "Show All Automated And Modulated Sliders",
  invoke = function(repeated) show_automated_modulated(false) end
}

renoise.tool():add_menu_entry {
  name = ct_menu_name .. "Show All Automated And Modulated Sliders",
  invoke = function() show_automated_modulated(true) end,
  active = function() return menu_active(true) end
}

renoise.tool():add_keybinding {
  name = ct_keybind_name .. "Show All Automated And Modulated Sliders",
  invoke = function(repeated) show_automated_modulated(true) end
}

renoise.tool():add_menu_entry {
  name = at_menu_name .. "Hide All Sliders",
  invoke = function() hide_all(false) end,
  active = function() return menu_active(false) end
}

renoise.tool():add_keybinding {
  name = at_keybind_name .. "Hide All Sliders",
  invoke = function(repeated) hide_all(false) end
}

renoise.tool():add_menu_entry {
  name = ct_menu_name .. "Hide All Sliders",
  invoke = function() hide_all(true) end,
  active = function() return menu_active(true) end
}

renoise.tool():add_keybinding {
  name = ct_keybind_name .. "Hide All Sliders",
  invoke = function(repeated) hide_all(true) end
}

--[[ enable / disable menu entries ]]---------------------------------------]]--

function menu_active(current_track)

  if not current_track then
    for _, track in ipairs(renoise.song().tracks) do
      if #track.devices > 1 then
        return true
      end
    end
    return false
  else
    if #renoise.song().selected_track.devices > 1 then
      return true
    end
    return false
  end

end

--[[ shows all automated sliders ]]-----------------------------------------]]--

function show_automated(current_track, hide)

  if hide then
    hide_all(current_track)
  end

  if not current_track then
    for _, track in ipairs(renoise.song().tracks) do
      for _, device in ipairs(track.devices) do
        for _, parameter in ipairs(device.parameters) do
          if not parameter.is_automated then
            parameter.show_in_mixer = false
          else
            parameter.show_in_mixer = true
          end
        end
      end
    end
  else
    for _, device in ipairs(renoise.song().selected_track.devices) do
      for _, parameter in ipairs(device.parameters) do
        if not parameter.is_automated then
          parameter.show_in_mixer = false
        else
          parameter.show_in_mixer = true
        end
      end
    end
  end

end

--[[ shows all modulated sliders ]]-----------------------------------------]]--

function show_modulated(current_track, hide)

  local known_devices = table.create ({
    "*Hydra Device",
    "*Key-Tracking Device",
    "*LFO Device",
    "*Signal Follower",
    "*Velocity Device",
    "*XY Device"
  })

  if hide then
    hide_all(current_track)
  end

  for t, track in ipairs(renoise.song().tracks) do
    for _, device in ipairs(track.devices) do
      if known_devices:find(device.name) then
        for p, parameter in ipairs(device.parameters) do

          if parameter.name:find("Parameter") and
             device.parameters[p - 1].name:find("Effect") and
             device.parameters[p - 2].name:find("Track") then

            if parameter.value > -1 then

              local track_index = device.parameters[p - 2].value + 1
              local device_index = device.parameters[p - 1].value + 1
              local parameter_index = parameter.value + 1
              local d = nil

              if not current_track then

                if track_index == 0 then
                  d = track.devices[device_index]
                  d.parameters[parameter_index].show_in_mixer = true
                else
                  d = renoise.song().tracks[track_index].devices[device_index]
                  d.parameters[parameter_index].show_in_mixer = true
                end

              else

                if track_index == 0 and
                   t == renoise.song().selected_track_index then

                  track_index = renoise.song().selected_track_index
                end

                if track_index == renoise.song().selected_track_index then
                  d = renoise.song().selected_track.devices[device_index]
                  d.parameters[parameter_index].show_in_mixer = true
                end

              end
            end
          end
        end
      end
    end
  end

end

--[[ shows all automated and modulated sliders ]]---------------------------]]--

function show_automated_modulated(current_track)

  show_automated(current_track, true)
  show_modulated(current_track, false)

end

--[[ hides all sliders ]]---------------------------------------------------]]--

function hide_all(current_track)

  if not current_track then
    for _, track in ipairs(renoise.song().tracks) do
      for _, device in ipairs(track.devices) do
        for _, parameter in ipairs(device.parameters) do
          parameter.show_in_mixer = false
        end
      end
    end
  else
    for _, device in ipairs(renoise.song().selected_track.devices) do
      for _, parameter in ipairs(device.parameters) do
        parameter.show_in_mixer = false
      end
    end
  end

end

---------------------------------------------------------------------[[ EOF ]]--
